# vim:set ts=2 sw=2 sts=2 et:

class Edge(object):
  def __init__(self, x1, y1, x2, y2):
    if y1 > y2:
      x1,y1,x2,y2 = x2,y2,x1,y1
    self.x1,self.y1,self.x2,self.y2 = x1,y1,x2,y2

  def lines_to(self, edge2, space):
    # difference in y for edges
    e1dy = float(self.y2 - self.y1)
    e2dy = float(edge2.y2 - edge2.y1)
    if (e1dy == 0) or (e2dy == 0):
      return []

    # difference in x for edges
    e1dx = float(self.x2 - self.x1)
    e2dx = float(edge2.x2 - edge2.x1)

    # x interpolation factors
    factor1 = float(edge2.y1 - self.y1) / e1dy
    incr1 = 1.0 / e1dy
    factor2 = 0.0
    incr2 = 1.0 / e2dy

    # loop through the lines between the edges
    lines = []
    for y in range(edge2.y1, edge2.y2, space):
      x1 = self.x1 + int(e1dx * factor1)
      x2 = edge2.x1 + int(e2dx * factor2)
      lines.extend([x1,y,x2,y])
      factor1 += incr1 * space
      factor2 += incr2 * space
    return lines


def choose_edges(x1,y1,x2,y2,x3,y3):
  edges = [Edge(int(x1), int(y1), int(x2), int(y2)),
           Edge(int(x2), int(y2), int(x3), int(y3)),
           Edge(int(x3), int(y3), int(x1), int(y1))]
  # find longest y edge
  maxLength = 0
  edgeIdx = 0
  for i in range(3):
    length = edges[i].y2 - edges[i].y1
    if length > maxLength:
      maxLength = length
      edgeIdx = i

  return edges[edgeIdx], edges[(edgeIdx + 1) % 3], edges[(edgeIdx + 2) % 3]


def rasterise(tris, space):
  lines = []
  space /= 2
  for i in range(len(tris)/6):
    # draw spans between edges
    long1, short1, short2 = choose_edges(*tris[i*6:i*6+6])
    lines.extend(long1.lines_to(short1, space) + 
                 long1.lines_to(short2, space))
  return lines
