# vim:set ts=2 sw=2 sts=2 et:
from rzn.api import Plugin, Logger
import xml.etree.ElementTree as ETree
from path import PathParser
import re

log = Logger(__name__)

# FIXME: move to config
SVG_SCALE = 1000


def import_path(elt):
  return PathParser(elt.attrib.get("d", "")).parse()


def import_polygon(elt):
  points = elt.attrib.get("points", "")
  polypts = []
  for pt in points.split():
    x, y = map(float, pt.split(","))
    polypts.extend((x,y))
  return polypts


def import_elt(proj, elt):
  m = re.match("({.*})?(.*)", elt.tag)
  _, tag =  m.groups()
  polys = []
  if tag == "polygon":
    polys.append(import_polygon(elt))
  elif tag == "path":
    polys.extend(import_path(elt))
  else:
    log.warning("Unhandled SVG element: %s" % tag)

  for poly in polys:
    pts = [int(x * SVG_SCALE) for x in poly]
    if len(pts) > 6:
      proj.layout.polygon((0,0), pts)


def import_svg(proj, fn):
  root = ETree.parse(fn).getroot()
  for elt in root:
    import_elt(proj, elt)


class SvgImportPlugin(Plugin):
  def __init__(self):
    Plugin.__init__(self, "SVG", Plugin.IMPORT)

  def main(self, ui):
    fn = ui.showOpenFileDialog("Choose an SVG file", "SVG files (*.svg)")
    self.batchMain(ui.currentTab().project(), {"file":fn})

  def batchMain(self, proj, options):
    fn = options.get("file")
    if fn:
      import_svg(proj, fn)
