# vim:set ts=2 sw=2 sts=2 et:
from rzn.api import Plugin, Layers
from rzn.api.pluginoption import FloatOption, BoolOption
from rzn.api.units import mm

class BoardOutlinePlugin(Plugin):
  def __init__(self):
    Plugin.__init__(self, "Board Outline Generator")

  def board(self, proj, opts):
    width = opts["width"] * mm(1)
    height = opts["height"] * mm(1)
    cor = opts["corner"] * mm(1)
    drl = opts["drill"] * mm(1)
    edg = opts["edge"] * mm(1)

    proj.grid(1)

    # edges
    w2, h2 = width/2, height/2
    proj.layout.line((-w2+cor, -h2), (w2-cor, -h2), 0, Layers.Border)
    proj.layout.line((-w2+cor,  h2), (w2-cor,  h2), 0, Layers.Border)
    proj.layout.line((-w2, -h2+cor), (-w2, h2-cor), 0, Layers.Border)
    proj.layout.line(( w2, -h2+cor), ( w2, h2-cor), 0, Layers.Border)
    # corner arcs
    if cor > 0:
      proj.layout.arc((-w2+cor, -h2+cor), (-cor,0), (0,-cor), 0, Layers.Border)
      proj.layout.arc(( w2-cor, -h2+cor), (0,-cor), (cor, 0), 0, Layers.Border)
      proj.layout.arc(( w2-cor,  h2-cor), (cor, 0), (0, cor), 0, Layers.Border)
      proj.layout.arc((-w2+cor,  h2-cor), (0, cor), (-cor,0), 0, Layers.Border)
    # mounting holes
    if drl > 0:
      proj.layout.drill((-w2+edg, -h2+edg), drl)
      proj.layout.drill(( w2-edg, -h2+edg), drl)
      proj.layout.drill(( w2-edg,  h2-edg), drl)
      proj.layout.drill((-w2+edg,  h2-edg), drl)
    # ground plane
    if opts["ground"]:
      p = proj.layout.polygon((0,0), (-w2-mm(1),-h2-mm(1),w2+mm(1),-h2-mm(1),
                                       w2+mm(1), h2+mm(1),-w2-mm(1),h2+mm(1)))
      p.layer = Layers.CopperBottom
      p.net = "gnd"

  def main(self, ui):
    options = [
        ("Width (mm)", "width", FloatOption(50)),
        ("Height (mm)", "height", FloatOption(31)),
        ("Corner radius (mm)", "corner", FloatOption(4)),
        ("Drill radius (mm)", "drill", FloatOption(3.2)),
        ("Drill to edge (mm)", "edge", FloatOption(4)),
        ("Ground plane", "ground", BoolOption(True)),
    ]
    presets = [
      ("Dangerous Prototypes 5031", [50, 31, 4, 3.2, 4, True]),
      ("Dangerous Prototypes 6037", [60, 37, 4, 3.2, 4, True]),
      ("Dangerous Prototypes 7043", [70, 43, 4, 3.2, 4, True]),
      ("Dangerous Prototypes 8049", [80, 49, 4, 3.2, 4, True]),
      ("Dangerous Prototypes 9056", [90, 56, 4, 3.2, 4, True]),
    ]
    opts = ui.showScriptOptionForm(options, presets)
    if opts:
      project = ui.currentTab().project()
      self.board(project, opts)
