# vim:set ts=2 sw=2 sts=2 et:
from rzn.api import GeneratorPlugin, Layers
from rzn.api.pluginoption import FloatOption, BoolOption
from rzn.api.units import mm, inch
from rzn.api.common import addrect

# width height space
presets = [
  ("1005 (0402 Imperial)", [0.6, 0.6 , 0.5, False]),
  ("1608 (0603 Imperial)", [0.8, 1.0 , 0.9, False]),
  ("2012 (0805 Imperial)", [0.9, 1.4 , 1.1, False]),
  ("3216 (1206 Imperial)", [1.2, 1.7 , 1.7, False]),
  ("3225 (1210 Imperial)", [1.2, 2.6 , 1.7, False]),
]

class SMDRCLPlugin(GeneratorPlugin):
  def __init__(self):
    GeneratorPlugin.__init__(self, "SMD Passive", [
      ("Pad width (mm)",   "padw",      FloatOption(0.6)),
      ("Pad height (mm)",  "padh",      FloatOption(0.6)),
      ("Pad spacing (mm)", "space",     FloatOption(0.5)),
      ("Polarised",        "polarised", BoolOption(False)),
    ], presets)

  def footprint(self, project, options):
    padw, padh, padspace = options["padw"], options["padh"], options["space"]
    project.grid(1)
    padw *= mm(1)
    padh *= mm(1)
    padspace *= mm(1)

    # add pads
    padofs = (padw+padspace)/2.0
    p=project.pad((padofs,0), padw, padh, "rectangle")
    p.name = "a"
    p=project.pad((-padofs,0), padw, padh, "rectangle")
    p.name = "b"

    bodyw = padw*2 + padspace + mm(0.5)
    bodyh = padh + mm(0.5)

    # add body outline
    addrect(project.layout.line, bodyw/2.0, bodyh/2.0, mm(0.25), Layers.OutlineTop)
    # polarity marker
    if options["polarised"]:
      project.layout.arc((-bodyw/2.0-mm(0.3), -bodyh/2.0-mm(0.3)),
                         (0,-mm(0.1)), (0,-mm(0.1)),
                         mm(0.25), Layers.OutlineTop)

    # name
    project.layout.text((-bodyw/2.0,-bodyh/2.0-mm(0.1)), "$name", 1, Layers.NameTop)

  def symbol(self, project, options):
    project.grid(inch(0.05))
    
    # add pins
    p = project.schematic.pin((5,0))
    p.name = "a"
    p.showname = False
    p = project.schematic.pin((-5,0))
    p.name = "b"
    p.angle = 180
    p.showname = False
    # add body outline
    addrect(project.schematic.line, 3, 1, 0, Layers.Symbol)
    # name
    project.schematic.text((-3,-1), "$name", 1, Layers.Label)
    # value
    project.schematic.text((-3,2), "$value", 1, Layers.Label)
