# vim:set ts=2 sw=2 sts=2 et:
from rzn.api import GeneratorPlugin, Layers
from rzn.api.pluginoption import IntOption, FloatOption
from rzn.api.units import mm, inch
from rzn.api.common import addrect

class SOICPlugin(GeneratorPlugin):
  def __init__(self):
    GeneratorPlugin.__init__(self, "SOIC", [
      ("Pin count",        "count",  IntOption(16)),
      ("Pin Pitch (inch)", "pitch",  FloatOption(0.05)),
      ("Pin width (inch)", "pinw",   FloatOption(0.06)),
      ("Width (inch)",     "width",  FloatOption(0.15)),
    ])

  def footprint(self, project, options):
    pins, width, pinwidth = options["count"], options["width"], options["pinw"]
    pitch = options["pitch"] * inch(1)
    width *= inch(1)
    pinwidth *= inch(1)
    pps = pins / 2 # pins per side
    bodyheight = (pps-1)*pitch + inch(0.05)
    po = ((pps-1)*pitch)/2.0
    project.grid(1)

    # add a rectangular pad
    def addpad(n, x, y):
      p=project.layout.pad((x,y), pinwidth, mm(0.6), "rectangle")
      p.name = "%d" % (n+1)

    # add pads
    for i in range(pps):
      addpad(i,       -(width+pinwidth)/2.0, i*pitch-po)
      addpad(pins-i-1, (width+pinwidth)/2.0, i*pitch-po)

    # add body outline with corner
    addrect(project.layout.line, width/2.0, bodyheight/2.0, mm(0.25), 
            Layers.OutlineTop, cs=inch(0.05))
    # name
    project.layout.text((-width/2.0,-bodyheight/2.0-mm(0.1)), 
                        "$name", 1, Layers.NameTop)

  def symbol(self, project, options):
    pins = options["count"]
    pps = pins / 2 # pins per side
    height = (pps+1)*2
    width = 16
    po = (height/2.0 - 2)
    project.grid(inch(0.05))
    
    # add a pin
    def addpin(n, x, y):
      p = project.schematic.pin((x,y))
      p.name = "%d" % (n+1)
      p.angle = 180 if x < 0 else 0

    # add pins
    for i in range(pps):
      addpin(i,       -width/2-2, i*2-po)
      addpin(pins-i-1, width/2+2, i*2-po)

    # add body outline
    addrect(project.schematic.line, width/2.0, height/2.0, 0, Layers.Symbol)
    # name
    project.schematic.text((-width/2.0,-height/2.0), "$name", 1, Layers.Label)
