# vim:set ts=2 sw=2 sts=2 et:
from rzn.api import GeneratorPlugin, Layers
from rzn.api.pluginoption import ListOption
from rzn.api.units import mm, inch
from rzn.api.common import addrect

# Package parameters (all in mm)
# package: (bodywidth, bodyheight, [pad, pad, ...])
# pad: (width, height, xpos, ypos)

packages = {
  "SOT223" : (3.3, 6.5, [
    (2.1, 1.0, -2.7, -2.286),
    (2.1, 1.0, -2.7,  0    ),
    (2.1, 1.0, -2.7,  2.286),
    (2.1, 3.5,  2.7,  0    ),
  ]),

  "SOT23-5" : (1.75, 3.1, [
    (1.0, 0.6, -1.2, -0.95),
    (1.0, 0.6, -1.2,  0   ),
    (1.0, 0.6, -1.2,  0.95),
    (1.0, 0.6,  1.2,  0.95),
    (1.0, 0.6,  1.2, -0.95),
  ]),
  
  "SOT23-6" : (1.75, 3.1, [
    (1.0, 0.6, -1.2, -0.95),
    (1.0, 0.6, -1.2,  0   ),
    (1.0, 0.6, -1.2,  0.95),
    (1.0, 0.6,  1.2,  0.95),
    (1.0, 0.6,  1.2,  0   ),
    (1.0, 0.6,  1.2, -0.95),
  ]),

  "SOT143" : (1.3, 2.9, [
    (1.5, 1.2, -1.1, -0.85),
    (1.5, 1.0, -1.1,  0.95),
    (1.5, 1.0,  1.1,  0.95),
    (1.5, 1.0,  1.1, -0.95),
  ]),
}

class SOTPlugin(GeneratorPlugin):
  def __init__(self):
    GeneratorPlugin.__init__(self, "SOT", [
      ("Package", "package", ListOption(["SOT223", "SOT23-5", 
                                         "SOT23-6", "SOT143"])),
    ])

  def footprint(self, project, options):
    bodywidth, bodyheight, pads = packages[options["package"]]
    project.grid(1)
    
    # add a rectangular pad
    def addpad(n, w, h, x, y):
      p=project.layout.pad((x,y), w, h, "rectangle")
      p.name = "%d" % (n+1)

    n = 0
    for w, h, x, y in pads:
      addpad(n, int(w*mm(1)), int(h*mm(1)), int(x*mm(1)), int(y*mm(1)))
      n += 1
    bodywidth *= mm(1)
    bodyheight *= mm(1)
    # add body outline with corner
    addrect(project.layout.line, bodywidth/2.0, bodyheight/2.0, mm(0.25), 
            Layers.OutlineTop, cs=inch(0.05))
    # name
    project.layout.text((-bodywidth/2.0,-bodyheight/2.0-mm(0.1)), 
                        "$name", 1, Layers.NameTop)

  def symbol(self, project, options):
    _, _, pads = packages[options["package"]]

    pins = len(pads)
    pps = int(round(pins / 2.)) # pins per side
    height = (pps+1)*2
    width = 16
    po = (height/2.0 - 2)
    project.grid(inch(0.05))
    
    # add a pin
    def addpin(n, x, y):
      p = project.schematic.pin((x,y))
      p.name = "%d" % (n+1)
      p.angle = 180 if x < 0 else 0

    # add pins
    for i in range(pps):
      addpin(i,       -width/2-2, i*2-po)
      if (pins-i-1) >= pps:
        addpin(pins-i-1, width/2+2, i*2-po)

    # add body outline
    addrect(project.schematic.line, width/2.0, height/2.0, 0, Layers.Symbol)
    # name
    project.schematic.text((-width/2.0,-height/2.0), "$name", 1, Layers.Label)
